@@ -18,6 +18,11 @@
 
 PXR_NAMESPACE_OPEN_SCOPE
 
+// This epsilon value is used in several places when evaluating Bezier curves.
+// Specifically in the calculation of the Bezier parameter that corresponds to a
+// given time. So this epsilon is being applied to a unitless bezier parameter
+// parameter that should be in the range [0..1].
+static constexpr double _parameterEpsilon = 1.0e-10;
 
 ////////////////////////////////////////////////////////////////////////////////
 // BEZIER MATH
@@ -91,26 +96,68 @@ namespace
     };
 }
 
-static double _FilterZeroes(
-    const std::vector<double> &candidates)
+// Filter the zeros, looking for the answer between 0 and 1.
+static double _FilterZeros(double z0, double z1, double z2)
 {
-    double result = 0;
-    bool found = false;
+    double result = z0;
+    double minError = std::abs(z0 - 0.5);
+    double error = std::abs(z1 - 0.5);
 
-    for (double c : candidates)
-    {
-        if (c >= 0 && c <= 1)
-        {
-            if (TF_VERIFY(!found || result == c,
-                          "Multiple zeros found!"))
-            {
-                result = c;
-                found = true;
-            }
+    if (error < minError) {
+        // Check for multiple zeros and warn of possible regressive spline.
+        if (minError < 0.5) {
+            // XXX: Possibly this should just be a TF_DEBUG message
+            TF_WARN("Possibly regressive spline. Zeros at %g, %g, and %g",
+                    z0, z1, z2);
         }
+        result = z1;
+        minError = error;
+    }
+
+    error = std::abs(z2 - 0.5);
+    if (error < minError) {
+        // Check for multiple zeros again and warn.
+        if (minError < 0.5) {
+            TF_WARN("Possibly regressive spline. Zeros at %g, %g, and %g.",
+                    z0, z1, z2);
+        }
+        result = z2;
+        minError = error;
+    }
+
+    if (minError > 0.5 + _parameterEpsilon) {
+        TF_WARN("No zero found in [0..1]. Zeros at %g, %g, and %g."
+                " Using %g.",
+                z0, z1, z2, result);
+    }
+
+    return result;
+}
+
+// Filter the zeros, looking for the answer between 0 and 1.
+static double _FilterZeros(double z0, double z1)
+{
+    double result = z0;
+    double minError = std::abs(z0 - 0.5);
+    double error = std::abs(z1 - 0.5);
+
+    if (error < minError) {
+        // Check for multiple zeros and warn of possible regressive spline.
+        if (minError < 0.5) {
+            // XXX: Possibly this should just be a TF_DEBUG message
+            TF_WARN("Possibly regressive spline. Zeros at %g and %g",
+                    z0, z1);
+        }
+        result = z1;
+        minError = error;
+    }
+
+    if (minError > 0.5 + _parameterEpsilon) {
+        TF_WARN("No zero found in [0..1]. Zeros at %g and %g."
+                " Using %g.",
+                z0, z1, result);
     }
 
-    TF_VERIFY(found, "No zeros found!");
     return result;
 }
 
@@ -127,7 +174,7 @@ static double _FindMonotonicZero(
     const double discrim = std::sqrt(std::pow(quad.b, 2) - 4 * quad.a * quad.c);
     const double root0 = (-quad.b - discrim) / (2 * quad.a);
     const double root1 = (-quad.b + discrim) / (2 * quad.a);
-    return _FilterZeroes({root0, root1});
+    return _FilterZeros(root0, root1);
 }
 
 // Finds the unique real t-value in [0, 1] that satisfies
@@ -157,7 +204,7 @@ static double _FindMonotonicZero(
         const double root1 = t1 * std::cos(phi/3) - b3;
         const double root2 = t1 * std::cos((phi + 2*M_PI) / 3) - b3;
         const double root3 = t1 * std::cos((phi + 4*M_PI) / 3) - b3;
-        return _FilterZeroes({root1, root2, root3});
+        return _FilterZeros(root1, root2, root3);
     }
     else if (discrim == 0)
     {
@@ -165,7 +212,7 @@ static double _FindMonotonicZero(
         const double u1 = -std::cbrt(q2);
         const double root1 = 2*u1 - b3;
         const double root2 = -u1 - b3;
-        return _FilterZeroes({root1, root2});
+        return _FilterZeros(root1, root2);
     }
     else
     {
@@ -192,6 +239,16 @@ static double _FindMonotonicZero(
 {
     // Fairly arbitrary tininess constant, not tuned carefully.
     // We can lose precision in some cases if this is too small or too big.
+    //
+    // XXX: Note: A better epsilon is probably something like:
+    //
+    //    epsilon = 1e-10 * (abs(a) + abs(b) + abs(c) + abs(d))
+    //
+    // This scales epsilon roughly by the range of time coordinates and means
+    // that the epsilon for a spline using nanoseconds is very different than
+    // one using seconds (as it should be). But we should put some computational
+    // mathimatical rigor behind that decision before changing it. Leaving it
+    // for now.
     static constexpr double epsilon = 1e-10;
 
     // Check for coefficients near zero.
@@ -225,6 +282,52 @@ static double _FindMonotonicZero(
         cubic.d / cubic.a);
 }
 
+// Finds and returns the normalized parameter t (in the range [0..1]) along a
+// cubic Bezier curve defined by beginData and endData at which the Bezier
+// curve's time reaches the given time.
+//
+// Note that if t is  outside [0..1] due to floating-point precision, it will
+// be clamped. If it is more than slightly outside the range, a warning will
+// be emitted.
+static double
+_FindBezierParameter(
+    const Ts_TypedKnotData<double> &beginData,
+    const Ts_TypedKnotData<double> &endData,
+    const TsTime time)
+{
+    // Find the coefficients for x = f(t).
+    // Offset everything by the eval time, so that we can just find a zero.
+    const _Cubic timeCubic = _Cubic::FromPoints(
+        beginData.time - time,
+        beginData.time + beginData.GetPostTanWidth() - time,
+        endData.time - endData.GetPreTanWidth() - time,
+        endData.time - time);
+
+    // Find the value of t for which f(t) = 0.
+    // Due to the offset, this is the t-value at which we reach the eval time.
+    double t = _FindMonotonicZero(timeCubic);
+
+    if (t < 0)
+    {
+        if (t < -_parameterEpsilon) {
+            TF_WARN("Bezier spline parameter t < -epsilon, t=%g, -epsilon=%g",
+                    t, -_parameterEpsilon);
+        }
+        t = 0;
+    }
+    else if (t > 1)
+    {
+        if (t > 1 + _parameterEpsilon) {
+            TF_WARN("Bezier spline parameter t > 1 + epsilon, t=%g,"
+                    " 1 + epsilon=%g",
+                    t, 1 + _parameterEpsilon);
+        }
+        t = 1;
+    }
+
+    return t;
+}
+
 static double
 _EvalBezier(
     const Ts_TypedKnotData<double> &beginDataIn,
@@ -239,30 +342,7 @@ _EvalBezier(
     Ts_RegressionPreventerBatchAccess::ProcessSegment(
         &beginData, &endData, TsAntiRegressionKeepRatio);
 
-    // Find the coefficients for x = f(t).
-    // Offset everything by the eval time, so that we can just find a zero.
-    const _Cubic timeCubic = _Cubic::FromPoints(
-        beginData.time - time,
-        beginData.time + beginData.GetPostTanWidth() - time,
-        endData.time - endData.GetPreTanWidth() - time,
-        endData.time - time);
-
-    // Find the value of t for which f(t) = 0.
-    // Due to the offset, this is the t-value at which we reach the eval time.
-    const double t = _FindMonotonicZero(timeCubic);
-
-    // t should always be in [0, 1], but tolerate some slight imprecision.
-    static constexpr double epsilon = 1e-10;
-    if (t <= 0)
-    {
-        TF_VERIFY(t > -epsilon);
-        return beginData.value;
-    }
-    else if (t >= 1)
-    {
-        TF_VERIFY(t < 1 + epsilon);
-        return endData.value;
-    }
+    double t = _FindBezierParameter(beginData, endData, time);
 
     // Find the coefficients for y = f(t).
     const _Cubic valueCubic = _Cubic::FromPoints(
@@ -280,6 +360,13 @@ _EvalBezier(
     {
         // Evaluate dy/dx (value delta over time delta)
         // as dy/dt / dx/dt (quotient of derivatives).
+
+        const _Cubic timeCubic = _Cubic::FromPoints(
+            beginData.time,
+            beginData.time + beginData.GetPostTanWidth(),
+            endData.time - endData.GetPreTanWidth(),
+            endData.time);
+
         const _Quadratic valueDeriv = valueCubic.GetDerivative();
         const _Quadratic timeDeriv = timeCubic.GetDerivative();
         return valueDeriv.Eval(t) / timeDeriv.Eval(t);
@@ -567,13 +654,17 @@ namespace
         // Knot copiers for special cases.
         void ReplaceBoundaryKnots(
             Ts_TypedKnotData<double> *prevData,
-            Ts_TypedKnotData<double> *nextData) const;
+            Ts_TypedKnotData<double> *nextData,
+            bool* generatedPrev = nullptr,
+            bool* generatedNext = nullptr) const;
         void ReplacePreExtrapKnots(
             Ts_TypedKnotData<double> *nextData,
-            Ts_TypedKnotData<double> *nextData2) const;
+            Ts_TypedKnotData<double> *nextData2,
+            bool* generatedNext = nullptr) const;
         void ReplacePostExtrapKnots(
             Ts_TypedKnotData<double> *prevData,
-            Ts_TypedKnotData<double> *prevData2) const;
+            Ts_TypedKnotData<double> *prevData2,
+            bool* generatedPrev = nullptr) const;
 
     private:
         void _ResolveInner();
@@ -821,6 +912,12 @@ void _LoopResolver::_ResolveInner()
                     _firstInnerProtoIndex, -lp.numPreLoops + 1);
             }
         }
+        // TODO: Handle the "second time looped" case.  There is an issue where
+        // the first knot in the spline is not generated by inner looping, but
+        // the second knot is. In this case, extrapolation using TsExtrapLinear
+        // should compute the linear slope using the first knot and the
+        // generated second knot, but it appears that we are not yet handling
+        // that case.
     }
 
     // Case 3: post-extrapolating, and the last knots are copies mad by inner
@@ -843,6 +940,9 @@ void _LoopResolver::_ResolveInner()
             _extrapKnot2 = _CopyProtoKnotData(
                 lastProtoIndex, lp.numPostLoops);
         }
+        // TODO: Handle "penultimate time looped" case. See the "second time
+        // looped" comment just above. The same logic applies at the end of the
+        // spline.
     }
 
     // Case 4: between last knot before looping region and start of looping
@@ -990,7 +1090,7 @@ void _LoopResolver::_DoExtrap(
     // the value offset in the opposite direction from the evaluation time,
     // because we first hop forward to evaluate, then apply the value offset
     // backward to obtain the value at the original time.
-    if (_data->preExtrapolation.mode == TsExtrapLoopRepeat
+    if (extrapolation.mode == TsExtrapLoopRepeat
         && _aspect != Ts_EvalDerivative)
     {
         _ComputeExtrapValueOffset();
@@ -999,9 +1099,8 @@ void _LoopResolver::_DoExtrap(
 
     // Oscillate mode: every other extrapolating loop iteration is reflected
     // in time.
-    else if (
-        _data->preExtrapolation.mode == TsExtrapLoopOscillate
-        && iterHop % 2 != 0)
+    else if (extrapolation.mode == TsExtrapLoopOscillate
+             && iterHop % 2 != 0)
     {
         _evalTime = _firstTime + (protoSpan - (_evalTime - _firstTime));
         _location = (_location == Ts_EvalPre ? Ts_EvalPost : Ts_EvalPre);
@@ -1061,10 +1160,19 @@ void _LoopResolver::_ComputeExtrapValueOffset()
 //
 void _LoopResolver::ReplaceBoundaryKnots(
     Ts_TypedKnotData<double> *prevData,
-    Ts_TypedKnotData<double> *nextData) const
+    Ts_TypedKnotData<double> *nextData,
+    bool* generatedPrev /* = nullptr */,
+    bool* generatedNext /* = nullptr */) const
 {
     const TsLoopParams &lp = _data->loopParams;
 
+    if (generatedPrev) {
+        *generatedPrev = false;
+    }
+    if (generatedNext) {
+        *generatedNext = false;
+    }
+
     // Case 1: between last prototype knot and prototype end, after performing
     // shift out of echo region, if any.  Make a copy of the first prototype
     // knot at the end of the prototype region, and use that as nextData.
@@ -1072,6 +1180,9 @@ void _LoopResolver::ReplaceBoundaryKnots(
     {
         *nextData = _CopyProtoKnotData(
             _firstInnerProtoIndex, 1);
+        if (generatedNext) {
+            *generatedNext = true;
+        }
     }
 
     // Case 2: between last knot before looping region and start of looping
@@ -1081,6 +1192,9 @@ void _LoopResolver::ReplaceBoundaryKnots(
     {
         *nextData = _CopyProtoKnotData(
             _firstInnerProtoIndex, -lp.numPreLoops);
+        if (generatedNext) {
+            *generatedNext = true;
+        }
     }
 
     // Case 3: between end of looping region and first knot after looping
@@ -1090,29 +1204,50 @@ void _LoopResolver::ReplaceBoundaryKnots(
     {
         *prevData = _CopyProtoKnotData(
             _firstInnerProtoIndex, lp.numPostLoops + 1);
+        if (generatedPrev) {
+            *generatedPrev = true;
+        }
     }
 }
 
 void _LoopResolver::ReplacePreExtrapKnots(
     Ts_TypedKnotData<double>* const nextData,
-    Ts_TypedKnotData<double>* const nextData2) const
+    Ts_TypedKnotData<double>* const nextData2,
+    bool* generatedNext /* = nullptr */) const
 {
-    if (!_firstTimeLooped)
+    if (!_firstTimeLooped) {
+        if (generatedNext) {
+            *generatedNext = false;
+        }
         return;
+    }
 
     *nextData = _extrapKnot1;
     *nextData2 = _extrapKnot2;
+
+    if (generatedNext) {
+        *generatedNext = true;
+    }
 }
 
 void _LoopResolver::ReplacePostExtrapKnots(
     Ts_TypedKnotData<double>* const prevData,
-    Ts_TypedKnotData<double>* const prevData2) const
+    Ts_TypedKnotData<double>* const prevData2,
+    bool* generatedPrev /* = nullptr */) const
 {
-    if (!_lastTimeLooped)
+    if (!_lastTimeLooped) {
+        if (generatedPrev) {
+            *generatedPrev = false;
+        }
         return;
+    }
 
     *prevData = _extrapKnot1;
     *prevData2 = _extrapKnot2;
+
+    if (generatedPrev) {
+        *generatedPrev = true;
+    }
 }
 
 Ts_TypedKnotData<double>
@@ -1477,5 +1612,605 @@ Ts_Eval(
         * (loopRes.GetNegate() ? -1 : 1);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// BREAKDOWN Methods
+static bool
+_BreakdownHermite(
+    const Ts_SplineData* const data,
+    Ts_TypedKnotData<double>& prevData,
+    Ts_TypedKnotData<double>& knotData,
+    Ts_TypedKnotData<double>& nextData)
+{
+    // Hermites are easy to breakdown. The new knot simply has the evaluated
+    // value and slope of the existing spline.
+    double value, slope;
+    
+    value = _EvalHermite(prevData, nextData, knotData.time, Ts_EvalValue);
+    slope = _EvalHermite(prevData, nextData, knotData.time, Ts_EvalDerivative);
+
+    knotData.value = value;
+    knotData.preTanSlope = slope;
+    knotData.postTanSlope = slope;
+
+    // Turn off any tangent algorithms to keep them from messing things up.
+    prevData.postTanAlgorithm = TsTangentAlgorithmNone;
+    knotData.preTanAlgorithm = TsTangentAlgorithmNone;
+    knotData.postTanAlgorithm = TsTangentAlgorithmNone;
+    nextData.preTanAlgorithm = TsTangentAlgorithmNone;
+
+    return true;
+}
+
+static bool
+_BreakdownBezier(
+    const Ts_SplineData* const data,
+    Ts_TypedKnotData<double>& prevData,
+    Ts_TypedKnotData<double>& knotData,
+    Ts_TypedKnotData<double>& nextData)
+{
+    // Bezier curves are easy to breakdown if you know the parametric point at
+    // which you wish to breakdown them. To find that we have to solve for the u
+    // value at which the time curve reaches knotData.time.
+
+    // If the segment is regressive, de-regress it.  Our eval-time behavior
+    // always uses the Keep Ratio strategy. Note that this may change prevData
+    // and nextData. This is fine since we're changing their tangents anyway
+    // once we breakdown the segment.
+    Ts_RegressionPreventerBatchAccess::ProcessSegment(
+        &prevData, &nextData, TsAntiRegressionKeepRatio);
+
+    double u = _FindBezierParameter(prevData, nextData, knotData.time);
+
+    // Get the Bezier control points.  Note that there is similar code in
+    // _Sampler::_SubdivideBezier in sample.cpp
+    GfVec2d cp[4] = {
+        {prevData.time, prevData.value},
+        {prevData.time + prevData.postTanWidth, 
+         prevData.value + prevData.GetPostTanHeight()},
+        {nextData.time - nextData.preTanWidth,
+         nextData.value + nextData.GetPreTanHeight()},
+        {nextData.time, nextData.value}};
+
+    // Run one De Casteljau interpolation
+    GfVec2d cp01   = GfLerp(u, cp[0], cp[1]);
+    GfVec2d cp12   = GfLerp(u, cp[1], cp[2]);
+    GfVec2d cp23   = GfLerp(u, cp[2], cp[3]);
+
+    GfVec2d cp012  = GfLerp(u, cp01, cp12);
+    GfVec2d cp123  = GfLerp(u, cp12, cp23);
+
+    GfVec2d cp0123 = GfLerp(u, cp012, cp123);
+
+    GfVec2d prevCp[4] = {cp[0], cp01, cp012, cp0123};
+    GfVec2d nextCp[4] = {cp0123, cp123, cp23, cp[3]};
+
+    // Rebuild the knot values.
+    GfVec2d prevDataPostTan = prevCp[1] - prevCp[0];
+    GfVec2d knotDataPreTan  = prevCp[3] - prevCp[2];
+    GfVec2d knotDataPostTan = nextCp[1] - nextCp[0];
+    GfVec2d nextDataPreTan  = nextCp[3] - nextCp[2];
+
+    // Since nothing was vertical going in, nothing should be vertical coming
+    // out, but someone could try to breakdown a Bezier at a time that is very,
+    // very, very close to one of the end points and we'd get some numerical
+    // instability. Or perhaps a chemist is animating something in
+    // femtoseconds...
+    //
+
+    // Convert a tangent from a direction vector to a width, slope
+    auto _ConvertTangent =
+        [](const GfVec2d& tangent) -> GfVec2d
+        {
+            // Tweak non-zero vertical tangents slightly off vertical so we can
+            // compute a slope. Guessing that 1 arc second (1/3600 of 1 degree)
+            // is a safe value. We're going to approximate that by 5.0e-6, so a
+            // near vertical slope is 1.0 / 5.0e-6 or 200000
+            constexpr double nearVertical = 200000.0;
+
+            GfVec2d result;
+
+            if (tangent[0] == 0) {
+                if (tangent[1] == 0) {
+                    // Zero length tangent. Slope could be any value so pick 0.
+                    // Width can also safely be 0
+                    result[0] = 0.0;
+                    result[1] = 0.0;
+                } else {
+                    // Vertical but non-zero length tangent. Rotate it slightly
+                    // approximating sin(angle) == angle and cos(angle) == 1.
+                    result[0] = std::abs(tangent[1]) / nearVertical;
+                    result[1] = std::copysign(nearVertical, tangent[1]);
+                }
+            } else {
+                result[0] = tangent[0];
+                result[1] = tangent[1] / tangent[0];
+            }
+            return result;
+        };
+
+    // Convert our tangent vectors into (width, slope) tuples
+    prevDataPostTan = _ConvertTangent(prevDataPostTan);
+    knotDataPreTan = _ConvertTangent(knotDataPreTan);
+    knotDataPostTan = _ConvertTangent(knotDataPostTan);
+    nextDataPreTan = _ConvertTangent(nextDataPreTan);
+    
+    prevData.postTanWidth = prevDataPostTan[0];
+    prevData.postTanSlope = prevDataPostTan[1];
+
+    knotData.preTanWidth = knotDataPreTan[0];
+    knotData.preTanSlope = knotDataPreTan[1];
+
+    knotData.value = cp0123[1];
+
+    knotData.postTanWidth = knotDataPostTan[0];
+    knotData.postTanSlope = knotDataPostTan[1];
+
+    nextData.preTanWidth = nextDataPreTan[0];
+    nextData.preTanSlope = nextDataPreTan[1];
+
+    // Turn off any tangent algorithms to keep them from messing things up.
+    prevData.postTanAlgorithm = TsTangentAlgorithmNone;
+    knotData.preTanAlgorithm = TsTangentAlgorithmNone;
+    knotData.postTanAlgorithm = TsTangentAlgorithmNone;
+    nextData.preTanAlgorithm = TsTangentAlgorithmNone;
+
+    // If the segment was not regressive then the split pieces should not be
+    // either, but if the original segment was on the edge of being regressive
+    // then round-off errors may make one of the new segments ever so slightly
+    // regressive. Rather than handling that at each evaluation, handle it now.
+    Ts_RegressionPreventerBatchAccess::ProcessSegment(
+        &prevData, &knotData, TsAntiRegressionKeepRatio);
+    Ts_RegressionPreventerBatchAccess::ProcessSegment(
+        &knotData, &nextData, TsAntiRegressionKeepRatio);
+
+    return true;
+}
+
+// Interpolate between two knots.
+//
+static bool
+_BreakdownBetweenKnots(
+    const Ts_SplineData* const data,
+    Ts_TypedKnotData<double>& prevData,
+    Ts_TypedKnotData<double>& knotData,
+    Ts_TypedKnotData<double>& nextData)
+{
+    switch (prevData.nextInterp) {
+      case TsInterpValueBlock:
+        knotData.nextInterp = TsInterpValueBlock;
+        return true;
+
+      case TsInterpHeld:
+        knotData.nextInterp = TsInterpHeld;
+        knotData.value = prevData.value;
+        return true;
+
+      case TsInterpLinear:
+        {
+            const double slope = _GetSegmentSlope(prevData, nextData);
+            knotData.nextInterp = TsInterpLinear;
+            knotData.value = _ExtrapolateLinear(prevData, slope,
+                                                knotData.time, Ts_EvalPost);
+            return true;
+        }
+
+      case TsInterpCurve:
+        knotData.nextInterp = TsInterpCurve;
+        if (data->curveType == TsCurveTypeBezier)
+        {
+            return _BreakdownBezier(data, prevData, knotData, nextData);
+        }
+        else
+        {
+            return _BreakdownHermite(data, prevData, knotData, nextData);
+        }
+    }
+
+    return false;
+}
+
+bool
+_BreakdownMain(
+    Ts_SplineData* const data,
+    const TsTime atTime,
+    const bool testOnly,
+    const _LoopResolver &loopRes,
+    GfInterval* affectedIntervalOut,
+    std::string* reason)
+{
+    size_t idx;
+    // Get a handy alias to the times array.
+    const std::vector<TsTime> &times = data->times;
+
+    const bool haveInnerLoops = data->HasInnerLoops();
+    const bool haveMultipleKnots =
+        haveInnerLoops || (times.size() > 1);
+
+    // For now, we're not allowing breakdown on a time that is either masked
+    // by inner looping or is in the extrapolated looping section.
+    //
+    // A breakdown in the region masked by inner looping would have no effect
+    // and a breakdown in an extrapolated looping section would have widespread
+    // effect.
+    //
+    // You can breakdown in an inner looping prototype or in an extrapolated
+    // region that is not looping.
+
+    // If we're at a time that's masked by inner looping, do not breakdown.
+    GfInterval protoInterval, loopedInterval;
+    if (haveInnerLoops) {
+        protoInterval = data->loopParams.GetPrototypeInterval();
+        loopedInterval = data->loopParams.GetLoopedInterval();
+        if (loopedInterval.Contains(atTime) &&
+            !protoInterval.Contains(atTime))
+        {
+            *reason = TfStringPrintf(
+                "Cannot breakdown a spline in a region masked by inner looping"
+                " (time=%g).", atTime);
+            if (!testOnly) {
+                TF_WARN(*reason);
+            }
+
+            return false;
+        }
+    }
+
+    GfInterval knotInterval(times.front(), times.back());
+    if (haveInnerLoops) {
+        // Union in the looped interval.
+        knotInterval |= loopedInterval;
+    }
+
+    // Or, if we're in a looped extrapolation region then we cannot breakdown
+    // without significantly changing the shape of the curve.
+    if (haveMultipleKnots
+        && ((atTime < knotInterval.GetMin()
+             && data->preExtrapolation.IsLooping())
+            || (atTime > knotInterval.GetMax()
+                && data->postExtrapolation.IsLooping())))
+    {
+        *reason = TfStringPrintf(
+            "Cannot breakdown a spline in a region generated by extrapolation"
+            " looping (time=%g).", atTime);
+        if (!testOnly) {
+            TF_WARN(*reason);
+        }
+
+        return false;
+    }
+    
+
+    // _LoopResolver is for resolving loops. It generally resolves them by
+    // mapping a requested input time to an eval time that is somewhere in the
+    // loop prototype. So if the computed eval time is different than the
+    // requested time, we have requested a time that is generated by looping and
+    // breakdown is not available.
+    //
+    // These checks were done explicity above so it should not be possible to
+    // fail this test. The test is here mainly to protect against future code
+    // changes that may challenge these assumptions.
+    if (!TF_VERIFY(loopRes.GetEvalTime() == atTime,
+                   "Verification failure: Cannot breakdown a spline in a looped"
+                   " region at time = %g. Please report a bug.", atTime))
+    {
+        *reason = TfStringPrintf(
+            "Verification failure: Cannot breakdown a spline in a looped"
+            " region at time = %g. Please report a bug.", atTime);
+            
+        return false;
+    }
+
+    // Track whether the prev and next knots are real knots in the spline or
+    // are generated by looping.
+    bool generatedPrev, generatedNext;
+
+    // Use binary search to find first knot at or after the specified time.
+    const auto lbIt = std::lower_bound(times.begin(), times.end(), atTime);
+
+    // Short circuit if we are exactly at a knot time. If there's already
+    // a knot there then we're done.
+    const bool atKnot = (lbIt != times.end() && *lbIt == atTime);
+    if (atKnot) {
+        // There's already a knot at that time.
+        *reason = TfStringPrintf(
+            "Cannot breakdown a spline at an existing knot. (time=%g)",
+            atTime);
+        if (!testOnly) {
+            TF_WARN(*reason);
+        }
+
+        return false;
+    }
+
+    // If we passed all the above tests, the breakdown should succeed.
+    if (testOnly) {
+        return true;
+    }
+
+    // We're going to affect the current time.
+    *affectedIntervalOut = GfInterval(atTime);
+
+    // Figure out where we are in the sequence.  Find the bracketing knots, the
+    // knot we're at, if any, and what type of position (before start, after
+    // end, at first knot, at last knot, at another knot, between knots).
+    const auto prevIt = (lbIt != times.begin() ? lbIt - 1 : times.end());
+    const auto nextIt = lbIt;
+    const bool beforeStart = (nextIt == times.begin());
+    const bool afterEnd = (!loopRes.IsBetweenLastProtoAndEnd() &&
+                           nextIt == times.end());
+
+    // Neighboring knot parameters.
+    Ts_TypedKnotData<double> knotData, prevData, nextData;
+    
+    if (prevIt != times.end())
+    {
+        prevData = data->GetKnotDataAsDouble(prevIt - times.begin());
+    }
+    if (nextIt != times.end())
+    {
+        nextData = data->GetKnotDataAsDouble(nextIt - times.begin());
+    }
+
+    // Data for the new knot. (We'll convert it to the right type after
+    // we figure out the values.
+    knotData.time = atTime;
+
+    // Extrapolate before first knot.
+    if (beforeStart)
+    {
+        // nextData is the first knot.  We also need the knot after that, if
+        // there is one.
+        Ts_TypedKnotData<double> nextData2;
+        if (nextIt + 1 != times.end())
+        {
+            nextData2 = data->GetKnotDataAsDouble((nextIt + 1) - times.begin());
+        }
+
+        loopRes.ReplacePreExtrapKnots(&nextData, &nextData2, &generatedNext);
+
+        if (data->preExtrapolation.mode == TsExtrapValueBlock) {
+            // Time is already set. Everything else default initialized.
+            // No need to change the next knot.
+            knotData.nextInterp = TsInterpValueBlock;
+            idx = data->SetKnotFromDouble(&knotData, VtDictionary());
+            data->UpdateKnotTangentsAtIndex(idx);
+            data->UpdateKnotTangentsAtIndex(idx + 1);
+            affectedIntervalOut->SetMax(nextData.time,
+                                        !generatedNext);  // maxClosed
+            return true;
+        }
+
+         // Find the extrapolation slope.
+        const std::optional<double> slope =
+            _GetExtrapolationSlope(
+                data->preExtrapolation,
+                haveMultipleKnots, nextData, nextData2,
+                Ts_EvalPre);
+
+        // No slope should only occur for value blocks, and value blocks were
+        // handled above.
+        if (!TF_VERIFY(slope,
+                       "Verification failure: Cannot compute pre-extrapolation"
+                       " slope for spline breakdown. Please report a bug."))
+        {
+            *reason = "Verification failure: Cannot compute pre-extrapolation"
+                      " slope for spline breakdown. Please report a bug.";
+            return false;
+        }
+
+        // Extrapolate value.
+        knotData.value = _ExtrapolateLinear(nextData, *slope, atTime,
+                                            Ts_EvalPre);
+
+        switch (data->preExtrapolation.mode)
+        {
+          case TsExtrapHeld:
+            knotData.nextInterp = TsInterpHeld;
+            break;
+
+          case TsExtrapLinear:
+          case TsExtrapSloped:
+            knotData.nextInterp = TsInterpLinear;
+            knotData.preTanSlope = knotData.postTanSlope = *slope;
+            break;
+
+          default:
+            // This covers TsExtrapLoopRepeat, TsExtrapLoopReset,
+            // TsExtrapLoopOscillate, TsExtrapValueBlock, and any other future
+            // extrapolation modes that were not handled above.  (They should
+            // all be handled above.)
+            *reason = TfStringPrintf(
+                "Verification failure: Invalid pre-extrapolation mode (%s)"
+                " when breaking down spline. Please report a bug.",
+                TfEnum::GetName(data->preExtrapolation.mode).c_str());
+            TF_CODING_ERROR(*reason);
+
+            return false;
+        }
+
+        // There's no prevData from which to compute a preTanWidth, so match the
+        // next knot instead.
+        knotData.preTanWidth = nextData.preTanWidth;
+        knotData.preTanAlgorithm = nextData.preTanAlgorithm;
+        knotData.postTanWidth = (nextData.time - knotData.time) / 3;
+        knotData.postTanAlgorithm = nextData.preTanAlgorithm;
+
+        // XXX: Think about propagating customData.
+        idx = data->SetKnotFromDouble(&knotData, VtDictionary());
+        data->UpdateKnotTangentsAtIndex(idx);
+        data->UpdateKnotTangentsAtIndex(idx + 1);
+
+        affectedIntervalOut->SetMax(nextData.time,
+                                    !generatedNext);  // maxClosed
+
+        return true;
+    }
+
+    // Extrapolate after last knot.
+    if (afterEnd)
+    {
+        // prevData is the last knot.  We also need the knot before that, if
+        // there is one.
+        Ts_TypedKnotData<double> prevData2;
+        if (prevIt != times.begin())
+        {
+            prevData2 = data->GetKnotDataAsDouble((prevIt - times.begin()) - 1);
+        }
+
+        loopRes.ReplacePostExtrapKnots(&prevData, &prevData2, &generatedPrev);
+
+        if (data->postExtrapolation.mode == TsExtrapValueBlock) {
+            // Time is already set. Everything else default initialized.
+            knotData.nextInterp = TsInterpValueBlock;
+            size_t idx = data->SetKnotFromDouble(&knotData, VtDictionary());
+            idx = data->UpdateKnotTangentsAtIndex(idx);
+            data->UpdateKnotTangentsAtIndex(idx);
+            *affectedIntervalOut = GfInterval(atTime);
+
+            // Ensure that prevData's interpolation type is value-block. Note
+            // that if prevData was generated by looping that changing the
+            // generated data will not change spline's data and the spline shape
+            // will change due to the inserted knot.
+            prevData.nextInterp = TsInterpValueBlock;
+            idx = data->SetKnotFromDouble(&prevData, VtDictionary());
+            data->UpdateKnotTangentsAtIndex(idx);
+            affectedIntervalOut->SetMin(prevData.time,
+                                        !generatedPrev);  // minClosed
+            
+            return true;
+        }
+
+        // Find the extrapolation slope.
+        const std::optional<double> slope =
+            _GetExtrapolationSlope(
+                data->postExtrapolation,
+                haveMultipleKnots, prevData, prevData2,
+                Ts_EvalPost);
+
+        // No slope should only occur for value blocks, and value blocks were
+        // handled above.
+        if (!TF_VERIFY(slope,
+                       "Cannot compute post-extrapolation slope in order to"
+                       " breakdown spline."))
+        {
+            *reason = "Coding error: Cannot compute post-extrapolation slope in"
+                      " order to breakdown spline.";
+            return false;
+        }
+
+        // Extrapolate value.
+        knotData.value = _ExtrapolateLinear(prevData, *slope, atTime,
+                                            Ts_EvalAtTime);
+
+        switch (data->postExtrapolation.mode)
+        {
+          case TsExtrapHeld:
+            knotData.nextInterp = TsInterpHeld;
+            if (prevData.nextInterp != TsInterpHeld) {
+                prevData.nextInterp = TsInterpHeld;
+                idx = data->SetKnotFromDouble(&prevData, VtDictionary());
+                data->UpdateKnotTangentsAtIndex(idx);
+            }
+            break;
+
+          case TsExtrapLinear:
+          case TsExtrapSloped:
+            knotData.nextInterp = TsInterpLinear;
+            knotData.preTanSlope = knotData.postTanSlope = *slope;
+            prevData.nextInterp = TsInterpLinear;
+            break;
+
+          default:
+            // This covers TsExtrapLoopRepeat, TsExtrapLoopReset,
+            // TsExtrapLoopOscillate, TsExtrapValueBlock, and any other future
+            // extrapolation modes that were not handled above.  (They should
+            // all be handled above.)
+            *reason = TfStringPrintf(
+                "Verification failure: Invalid post-extrapolation mode (%s)"
+                " when breaking down spline. Please report a bug.",
+                TfEnum::GetName(data->postExtrapolation.mode).c_str());
+            TF_CODING_ERROR(*reason);
+            return false;
+        }
+
+        // There's no nextKnot from which to compute a postTanWidth, so match
+        // the prev knot instead.
+        knotData.preTanWidth = (prevData.time - knotData.time) / 3;
+        knotData.preTanAlgorithm = prevData.postTanAlgorithm;
+        knotData.postTanWidth = prevData.postTanWidth;
+        knotData.postTanAlgorithm = prevData.postTanAlgorithm;
+
+        // XXX: Think about propagating customData.
+        idx = data->SetKnotFromDouble(&knotData, VtDictionary());
+        data->UpdateKnotTangentsAtIndex(idx);
+        data->UpdateKnotTangentsAtIndex(idx - 1);
+
+        affectedIntervalOut->SetMin(prevData.time,
+                                    !generatedPrev);  // minClosed
+
+        return true;
+    }
+
+    // Otherwise we are between knots.
+
+    // Account for loop-boundary cases.
+    loopRes.ReplaceBoundaryKnots(&prevData, &nextData,
+                                 &generatedPrev, &generatedNext);
+
+    
+    // Interpolate. Note that prevData, knotData, and nextData may be modified
+    // in place.
+    bool result = _BreakdownBetweenKnots(data, prevData, knotData, nextData);
+
+    if (result) {
+        if (!generatedPrev) {
+            data->SetKnotFromDouble(&prevData, VtDictionary());
+        }
+
+        idx = data->SetKnotFromDouble(&knotData, VtDictionary());
+
+        if (!generatedNext) {
+            data->SetKnotFromDouble(&nextData, VtDictionary());
+        }
+
+        data->UpdateKnotTangentsAtIndex(idx - 1);
+        data->UpdateKnotTangentsAtIndex(idx);
+        data->UpdateKnotTangentsAtIndex(idx + 1);
+
+        *affectedIntervalOut = GfInterval(prevData.time,
+                                          nextData.time,
+                                          !generatedPrev,   // minClosed
+                                          !generatedNext);  // maxClosed
+    }
+
+    return result;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// BREAKDOWN ENTRY POINT
+
+bool
+Ts_Breakdown(
+    Ts_SplineData* const data,
+    const TsTime atTime,
+    const bool testOnly,
+    GfInterval *affectedIntervalOut,
+    std::string* reason)
+{
+    // Cannot split an empty spline.
+    if (data->times.empty())
+    {
+        *reason = "Cannot breakdown an empty spline.";
+        return false;
+    }
+
+    // We use the same loop resolver as Ts_Eval so we can breakdown the presence
+    // of looping and special interpolation cases.
+    _LoopResolver loopRes(data, atTime, Ts_EvalValue, Ts_EvalAtTime);
+
+    return _BreakdownMain(data, atTime, testOnly, loopRes, affectedIntervalOut,
+                          reason);
+}
+
 
 PXR_NAMESPACE_CLOSE_SCOPE
