@@ -277,7 +277,8 @@ ARCH_CONSTRUCTOR(Vt_CastRegistryInit, 255)
 }
 
 bool
-VtValue::IsArrayValued() const {
+VtValue::IsArrayValued() const
+{
     if (IsEmpty()) {
         return false;
     }
@@ -287,6 +288,12 @@ VtValue::IsArrayValued() const {
     return _info->isArray;
 }
 
+bool
+VtValue::IsArrayEditValued() const
+{
+    return GetElementTypeid() != typeid(void) && !IsArrayValued();
+}
+
 const Vt_ShapeData*
 VtValue::_GetShapeData() const
 {
