@@ -28,10 +28,13 @@
 #include "pxr/usd/sdr/shaderProperty.h"
 #include "pxr/usd/plugin/sdrOsl/oslParser.h"
 
+#include <map>
+#include <optional>
 #include <tuple>
 
 PXR_NAMESPACE_OPEN_SCOPE
 
+using ShaderMetadataHelpers::CreateStringFromStringVec;
 using ShaderMetadataHelpers::IsPropertyAnAssetIdentifier;
 using ShaderMetadataHelpers::IsPropertyATerminal;
 using ShaderMetadataHelpers::IsTruthy;
@@ -44,6 +47,8 @@ TF_DEFINE_PRIVATE_TOKENS(
 
     ((arraySize, "arraySize"))
     ((pageStr, "page"))
+    ((pageOpenStr, "page_open"))
+    ((openStr, "open"))
     ((oslPageDelimiter, "."))
     ((vstructMember, "vstructmember"))
     (sdrDefinitionName)
@@ -96,6 +101,56 @@ _ParseFromSourceCode(OSL::OSLQuery* query, const String& sourceCode)
 #endif
 }
 
+static SdrStringVec
+_GetOpenPages(const SdrShaderPropertyUniquePtrVec& properties)
+{
+    // Extract open pages from parameter metadata (pages are considered to be
+    // closed by default, unless marked open). Look for parameters that specify
+    // both a page and a page open status. If all such parameters agree the
+    // page is open, add that page to the result vector.
+
+    // Map page name to open status
+    std::map<std::string, bool> openStatusMap;
+
+    for (const auto& prop : properties) {
+        const SdrTokenMap& metadata = prop->GetMetadata();
+        std::optional<std::string> page;
+        std::optional<bool> openState;
+
+        for (const auto& it : metadata) {
+            // Check for page and open metadata. Openness may be indicated by
+            // either "open" or "page_open".
+            if (it.first == _tokens->pageStr) {
+                page = it.second;
+            }
+            else if (it.first == _tokens->openStr) {
+                openState = IsTruthy(_tokens->openStr, metadata);
+            }
+            else if (it.first == _tokens->pageOpenStr) {
+                openState = IsTruthy(_tokens->pageOpenStr, metadata);
+            }
+        }
+
+        if (page && openState) {
+            // And-together all "open" values for prop's page, so it's only
+            // marked open if all props that have an opinion agree
+            auto result = openStatusMap.insert({ *page, *openState });
+            if (!result.second) {
+                result.first->second &= *openState;
+            }
+        }
+    }
+
+    SdrStringVec openPages;
+    for (const auto& it : openStatusMap) {
+        if (it.second) {
+            openPages.push_back(it.first);
+        }
+    }
+
+    return openPages;
+}
+
 SdrShaderNodeUniquePtr
 SdrOslParserPlugin::ParseShaderNode(
     const SdrShaderNodeDiscoveryResult& discoveryResult)
@@ -168,6 +223,17 @@ SdrOslParserPlugin::ParseShaderNode(
         fallbackPrefix = it->second;
     }
 
+    // Generate properties
+    SdrShaderPropertyUniquePtrVec properties =
+        _getNodeProperties(oslQuery, discoveryResult, fallbackPrefix);
+
+    // Populate open pages from property metadata
+    const SdrStringVec openPages = _GetOpenPages(properties);
+    if (!openPages.empty()) {
+        metadata[SdrNodeMetadata->OpenPages] =
+            CreateStringFromStringVec(openPages);
+    }
+
     return SdrShaderNodeUniquePtr(
         new SdrShaderNode(
             discoveryResult.identifier,
@@ -180,7 +246,7 @@ SdrOslParserPlugin::ParseShaderNode(
             discoveryResult.resolvedUri,    // Definitive assertion that the
                                             // implementation is the same asset
                                             // as the definition
-            _getNodeProperties(oslQuery, discoveryResult, fallbackPrefix),
+            std::move(properties),
             metadata,
             discoveryResult.sourceCode
         )
