@@ -8,22 +8,43 @@
 #include "pxr/usd/sdf/variableExpressionParser.h"
 
 #include "pxr/usd/sdf/debugCodes.h"
+#include "pxr/usd/sdf/variableExpressionAST.h"
 #include "pxr/usd/sdf/variableExpressionImpl.h"
 
 #include "pxr/base/pegtl/pegtl.hpp"
 #include "pxr/base/pegtl/pegtl/contrib/trace.hpp"
 #include "pxr/base/tf/stringUtils.h"
 
+#include <array>
 #include <tuple>
 
 using namespace PXR_PEGTL_NAMESPACE;
 
 PXR_NAMESPACE_OPEN_SCOPE
 
+namespace SdfVariableExpressionASTNodes
+{
+
+// Helper for calling private constructors on the various
+// SdfVariableExpressionASTNodes::Node subclasses.
+class _NodeCreator
+{
+public:
+    template <class NodeType, class... Args>
+    static auto MakeNode(Args&&... nodeArgs)
+    {
+        return std::unique_ptr<NodeType>(
+            new NodeType(std::forward<Args>(nodeArgs)...));
+    }
+};
+
+} // end namespace SdfVariableExpressionASTNodes
+
 namespace
 {
 
 namespace Impl = Sdf_VariableExpressionImpl;
+namespace ASTNodes = SdfVariableExpressionASTNodes;
 
 // Node creators -----------------------------------------------
 // These objects are responsible for saving intermediate values as
@@ -35,10 +56,41 @@ class NodeCreator
 public:
     virtual ~NodeCreator();
     virtual std::unique_ptr<Impl::Node> CreateNode(std::string* errMsg) = 0;
+    virtual std::unique_ptr<ASTNodes::Node> CreateASTNode(
+        std::string* errMsg) = 0;
 };
 
 NodeCreator::~NodeCreator() = default;
 
+static
+std::vector<std::unique_ptr<Impl::Node>>
+_CreateNodes(
+    const std::vector<std::unique_ptr<NodeCreator>>& creators,
+    std::string* errMsg)
+{
+    std::vector<std::unique_ptr<Impl::Node>> nodes;
+    nodes.reserve(creators.size());
+    for (auto& c : creators) {
+        nodes.push_back(c->CreateNode(errMsg));
+    }
+
+    return nodes;
+}
+
+static
+ASTNodes::NodeList
+_CreateASTNodeList(
+    const std::vector<std::unique_ptr<NodeCreator>>& creators,
+    std::string* errMsg)
+{
+    std::vector<std::unique_ptr<ASTNodes::Node>> nodes;
+    nodes.reserve(creators.size());
+    for (auto& c : creators) {
+        nodes.push_back(c->CreateASTNode(errMsg));
+    }
+    return ASTNodes::NodeList(std::move(nodes));
+}
+
 class StringNodeCreator
     : public NodeCreator
 {
@@ -47,6 +99,18 @@ public:
     {
         return std::make_unique<Impl::StringNode>(std::move(parts));
     }
+
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) override
+    {
+        std::string s;
+        for (const auto& part : parts) {
+            s += part.content;
+        }
+
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::LiteralNode>(
+            std::move(s));
+    }
     
     using Part = Impl::StringNode::Part;
     std::vector<Part> parts;
@@ -60,6 +124,13 @@ public:
     {
         return std::make_unique<Impl::VariableNode>(std::move(var));
     }
+
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) override
+    {
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::VariableNode>(
+            std::move(var));
+    }
     
     std::string var;
 };
@@ -73,6 +144,12 @@ public:
     {
         return std::make_unique<Impl::ConstantNode<Type>>(value);
     }
+
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) override
+    {
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::LiteralNode>(value);
+    }
     
     Type value;
 };
@@ -88,6 +165,12 @@ public:
     {
         return std::make_unique<Impl::NoneNode>();
     }
+
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) override
+    {
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::LiteralNode>();
+    }
 };
 
 class ListNodeCreator
@@ -96,10 +179,28 @@ class ListNodeCreator
 public:
     std::unique_ptr<Impl::Node> CreateNode(std::string* errMsg) final
     {
-        return std::make_unique<Impl::ListNode>(std::move(elements));
+        std::vector<std::unique_ptr<Impl::Node>> elemNodes = 
+            _CreateNodes(elements, errMsg);
+        if (!errMsg->empty()) {
+            return nullptr;
+        }
+
+        return std::make_unique<Impl::ListNode>(std::move(elemNodes));
     };
 
-    std::vector<std::unique_ptr<Impl::Node>> elements;
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) override
+    {
+        ASTNodes::NodeList astNodes = _CreateASTNodeList(elements, errMsg);
+        if (!errMsg->empty()) {
+            return nullptr;
+        }
+
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::ListNode>(
+            std::move(astNodes));
+    }
+
+    std::vector<std::unique_ptr<NodeCreator>> elements;
 };
 
 // List that defines the set of functions recognized by the expression parser.
@@ -140,8 +241,20 @@ public:
             errMsg, &matchedFunctionName, (AvailableFunctions*)(nullptr));
     }
 
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg) final
+    {
+        ASTNodes::NodeList astNodes = _CreateASTNodeList(functionArgs, errMsg);
+        if (!errMsg->empty()) {
+            return nullptr;
+        }
+
+        return ASTNodes::_NodeCreator::MakeNode<ASTNodes::FunctionNode>(
+            std::move(functionName), std::move(astNodes));
+    }
+
     std::string functionName;
-    std::vector<std::unique_ptr<Impl::Node>> functionArgs;
+    std::vector<std::unique_ptr<NodeCreator>> functionArgs;
 
 private:
     // Search the list of available function nodes for one whose name matches
@@ -199,7 +312,15 @@ private:
                 functionName.c_str(), NodeType::MinNumArgs);
             return nullptr;
         }
-        return std::make_unique<NodeType>(std::move(functionArgs));
+
+        std::vector<std::unique_ptr<Impl::Node>> argNodes =
+            _CreateNodes(functionArgs, errMsg);
+        if (!errMsg->empty()) {
+            return nullptr;
+        }
+        
+
+        return std::make_unique<NodeType>(std::move(argNodes));
     }
 
     // Construct an instance of NodeType if it's a function that requires a
@@ -220,7 +341,15 @@ private:
     std::unique_ptr<Impl::Node> _CreateFunctionNode(
         std::string* errMsg, std::index_sequence<I...>)
     { 
-        return std::make_unique<NodeType>(std::move(functionArgs[I])...);
+        std::array<std::unique_ptr<Impl::Node>, sizeof...(I)> argNodes = {
+            functionArgs[I]->CreateNode(errMsg)...
+        };
+
+        if (!errMsg->empty()) {
+            return nullptr;
+        }
+
+        return std::make_unique<NodeType>(std::move(argNodes[I])...);
     }
 };
 
@@ -231,6 +360,13 @@ private:
 class ParserContext
 {
 public:
+    ParserContext() = default;
+    ParserContext(const ParserContext&) = delete;
+    ParserContext(ParserContext&&) = default;
+
+    ParserContext& operator=(const ParserContext&) = delete;
+    ParserContext& operator=(ParserContext&&) = default;
+
     // Create and push a node creator of type CreatorType onto the stack,
     // passing in any given args to CreatorType c'tor, and return it.
     template <class CreatorType, class... Args>
@@ -240,6 +376,17 @@ public:
         return static_cast<CreatorType*>(_nodeStack.back().get());
     }
 
+    std::unique_ptr<NodeCreator> PopNodeCreator()
+    {
+        if (!TF_VERIFY(!_nodeStack.empty()) || !TF_VERIFY(_nodeStack.back())) {
+            return nullptr;
+        }
+
+        std::unique_ptr<NodeCreator> creator = std::move(_nodeStack.back());
+        _nodeStack.pop_back();
+        return creator;
+    }
+
     // Return pointer to the node creator on the top of the stack if
     // it is an instance of CreatorType, otherwise return nullptr.
     template <class CreatorType>
@@ -282,6 +429,22 @@ public:
         return creator->CreateNode(errMsg);
     }
 
+    // Pop the node creator from the top of the stack and use it to
+    // create an AST node.
+    std::unique_ptr<ASTNodes::Node>
+    CreateASTNode(std::string* errMsg)
+    {
+        if (!TF_VERIFY(!_nodeStack.empty()) || !TF_VERIFY(_nodeStack.back())) {
+            *errMsg = "Unknown error";
+            return nullptr;
+        }
+
+        std::unique_ptr<NodeCreator> creator = std::move(_nodeStack.back());
+        _nodeStack.pop_back();
+
+        return creator->CreateASTNode(errMsg);
+    }
+
 private:
     std::vector<std::unique_ptr<NodeCreator>> _nodeStack;
 };
@@ -686,20 +849,20 @@ struct Action<ListElement>
     template <typename ActionInput>
     static void apply(const ActionInput& in, ParserContext& context)
     {
-        std::string errMsg;
-        std::unique_ptr<Impl::Node> elemNode = 
-            context.CreateExpressionNode(&errMsg);
-        if (!elemNode) {
-            _ThrowParseError(in, errMsg);
+        std::unique_ptr<NodeCreator> elemCreator = context.PopNodeCreator();
+        if (!elemCreator) {
+            _ThrowParseError(in, 
+                "Internal error: could not pop node creator stack");
         }
 
         ListNodeCreator* listCreator =
             context.GetExistingNodeCreator<ListNodeCreator>();
         if (!listCreator) {
-            _ThrowParseError(in, "Unknown error");
+            _ThrowParseError(in,
+                "Internal error: list creator not at top of stack");
         }
 
-        listCreator->elements.push_back(std::move(elemNode));
+        listCreator->elements.push_back(std::move(elemCreator));
     }
 };
 
@@ -719,20 +882,20 @@ struct Action<FunctionArgument>
     template <typename ActionInput>
     static void apply(const ActionInput& in, ParserContext& context)
     {
-        std::string errMsg;
-        std::unique_ptr<Impl::Node> argNode = 
-            context.CreateExpressionNode(&errMsg);
-        if (!argNode) {
-            _ThrowParseError(in, errMsg);
+        std::unique_ptr<NodeCreator> elemCreator = context.PopNodeCreator();
+        if (!elemCreator) {
+            _ThrowParseError(in,
+                "Internal error: could not pop node creator stack");
         }
 
         FunctionNodeCreator* fnCreator =
             context.GetExistingNodeCreator<FunctionNodeCreator>();
         if (!fnCreator) {
-            _ThrowParseError(in, "Unknown error");
+            _ThrowParseError(in,
+                "Internal error: function creator not at top of stack");
         }
 
-        fnCreator->functionArgs.push_back(std::move(argNode));
+        fnCreator->functionArgs.push_back(std::move(elemCreator));
     }
 };
 
@@ -776,10 +939,16 @@ MATCH_ERROR(DoubleQuotedString::End, R"(Missing ending '"')");
 MATCH_ERROR(SingleQuotedString::Body, "Invalid string contents");
 MATCH_ERROR(SingleQuotedString::End, R"(Missing ending "'")");
 
+struct ParseResult
+{
+    std::optional<ParserContext> context;
+    std::vector<std::string> errMsg;
+};
+
 } // end anonymous namespace
 
-Sdf_VariableExpressionParserResult
-Sdf_ParseVariableExpression(const std::string& expr)
+static ParseResult
+Sdf_ParseVariableExpressionImpl(const std::string& expr)
 {
     namespace pegtl = PXR_PEGTL_NAMESPACE;
 
@@ -794,12 +963,12 @@ Sdf_ParseVariableExpression(const std::string& expr)
             pegtl::parse<Expression, Action, Errors>(in, context);
         
         if (!parseSuccess) {
-            return { nullptr, { "Unable to parse expression" } };
+            return { std::nullopt, { "Unable to parse expression" } };
         }
     }
     catch (const pegtl::parse_error& e) {
         return { 
-            nullptr, 
+            std::nullopt,
 
             // XXX: "at character" is probably incorrect if the expression
             // contains Unicode strings?
@@ -810,9 +979,20 @@ Sdf_ParseVariableExpression(const std::string& expr)
         };
      }
 
+    return { std::move(context), {} };
+}
+
+Sdf_VariableExpressionParserResult
+Sdf_ParseVariableExpression(const std::string& expr)
+{
+    ParseResult result = Sdf_ParseVariableExpressionImpl(expr);
+    if (!result.context) {
+        return { nullptr, std::move(result.errMsg) };
+    }
+
     std::string errMsg;
     std::unique_ptr<Impl::Node> exprNode = 
-        context.CreateExpressionNode(&errMsg);
+        result.context->CreateExpressionNode(&errMsg);
 
     if (!exprNode) {
         return { nullptr, { std::move(errMsg) } };
@@ -821,6 +1001,25 @@ Sdf_ParseVariableExpression(const std::string& expr)
     return { std::move(exprNode), {} };
 }
 
+Sdf_VariableExpressionASTParserResult
+Sdf_ParseVariableExpressionAST(const std::string& expr)
+{
+    ParseResult result = Sdf_ParseVariableExpressionImpl(expr);
+    if (!result.context) {
+        return { nullptr, std::move(result.errMsg) };
+    }
+
+    std::string errMsg;
+    std::unique_ptr<ASTNodes::Node> astNode = 
+        result.context->CreateASTNode(&errMsg);
+
+    if (!astNode) {
+        return { nullptr, { std::move(errMsg) } };
+    }
+
+    return { std::move(astNode), {} };
+}
+
 bool
 Sdf_IsVariableExpression(const std::string& s)
 {
